@@ -6,10 +6,19 @@
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include "efaxmsg.h"
 #include "efaxlib.h"
 
+#include <config.h>             /* For NLS */
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#include <glib/gconvert.h>
+#include <glib/gunicode.h>
+#include <glib/gmem.h>
+#endif
+
 #ifndef SEEK_SET
 #define SEEK_SET 0
 #endif
@@ -41,6 +50,31 @@ int ckfmt ( char *p, int n )
   return n < 0 ;
 }
 
+/* Provide dummy gettext() function if there is no internationalisation support */
+
+#ifndef ENABLE_NLS
+static const char *gettext ( const char *text )
+{
+  return text;
+}
+#endif
+
+char *strdup2 ( const char *str1, const char *str2 )
+{
+  int str1_len=0, str2_len=0 ;
+  char *out;
+
+  str1_len = strlen ( str1 ) ;
+  str2_len = strlen ( str2 ) ;
+
+  out = malloc ( str1_len + str2_len + 1 ) ;
+  if ( out ) {
+    memcpy ( out , str1, str1_len ) ;
+    memcpy ( out + str1_len, str2, str2_len + 1 ) ;
+  }
+
+  return out ;
+}
 
 /* Initialize state of variable-length code word encoder. */
 
@@ -434,10 +468,28 @@ void logifnames ( IFILE *f, char *s )
   PAGE *p ;
   char *fn ;
 
+#ifdef ENABLE_NLS
+  gsize written = 0 ;
+#endif
+
   if ( f && f->page )
     for ( p = f->page ; p <= f->lastpage ; p++ ) {
       fn = p->fname ;
+#ifdef ENABLE_NLS
+      if ( fn ) {
+	if ( use_utf8 && !g_utf8_validate ( fn, -1, NULL ) ) {
+	  fn = g_filename_to_utf8 ( fn, -1, NULL, &written, NULL ) ;
+	  if ( fn ) {
+	    msg ( s, fn ) ;
+	    g_free ( fn ) ;
+	  }
+	} else {
+	  msg ( s, fn ) ;
+	}
+      }
+#else
       if ( fn ) msg ( s, fn ) ;
+#endif
     }
 }
 
@@ -605,7 +657,7 @@ int readline ( IFILE *f, short *runs, int *pels )
       
     case P_PCX:
       nb = ( ( f->page->w + 15 ) / 16 ) * 2 ;	/* round up */
-      if ( readpcx ( bits, nb, f ) != 0 ) {
+      if ( readpcx ( (char*) bits, nb, f ) != 0 ) {
 	nr = EOF ;
       } else {
    	nr = bittorun ( bits, nb, runs ) ;
@@ -645,7 +697,7 @@ int getformat ( uchar *p, int n )
     format = I_FAX ;
   } 
 
-  if ( ! format && ! strncmp ( p, "P4", 2 ) ) {
+  if ( ! format && ! strncmp ( (char*) p, "P4", 2 ) ) {
     format = I_PBM ;
   }
 
@@ -658,7 +710,7 @@ int getformat ( uchar *p, int n )
     }
   }
 
-  if ( ! format && ! strncmp ( p, "%!", 2 ) ) {
+  if ( ! format && ! strncmp ( (char*) p, "%!", 2 ) ) {
     msg ( "W Postscript input file will be treated as text" ) ;
   }
 
@@ -823,8 +875,8 @@ int tiff_next ( IFILE *f )
       long a, b, where=0 ;
       err = err || ( ( where = ftell ( f->f ) ) < 0 ) ;
       err = err || fseek ( f->f, tv, SEEK_SET ) ;
-      err = err || fread4 ( &a, f ) ;
-      err = err || fread4 ( &b, f ) ;
+      err = err || fread4 ( (unsigned long*) &a, f ) ;
+      err = err || fread4 ( (unsigned long*) &b, f ) ;
       err = err || fseek ( f->f, where, SEEK_SET ) ;
       ftv = (float) a / ( b ? b : 1 ) ;
     } else { 
@@ -905,7 +957,7 @@ int tiff_next ( IFILE *f )
   
   if ( ! err ) {
 
-    if ( fread4 ( &(f->next), f ) ) {
+    if ( fread4 ( (unsigned long*) &(f->next), f ) ) {
       err = msg ( "E2can't read offset to next TIFF directory" ) ;
     } else {
       if ( f->next ) {
@@ -932,8 +984,8 @@ int tiff_first ( IFILE *f )
 
   fread ( (uchar*) &magic, 1, 2, f->f ) ;
   f->bigend = ( *(uchar*) &magic == 'M' ) ? 1 : 0 ;
-  fread2 ( &version, f ) ;
-  fread4 ( &(f->next), f ) ;
+  fread2 ( (unsigned short*) &version, f ) ;
+  fread4 ( (unsigned long*) &(f->next), f ) ;
   
   msg ( "F TIFF version %d.%d file (%s-endian)",
        version/10, version%10, f->bigend ? "big" : "little" ) ;
@@ -1081,8 +1133,8 @@ int dcx_next ( IFILE *f )
   /* get this and next pages' offsets */
 
   fseek ( f->f, f->next, SEEK_SET ) ; 
-  fread4 ( &thisp, f ) ;
-  fread4 ( &nextp, f ) ;
+  fread4 ( (unsigned long*) &thisp, f ) ;
+  fread4 ( (unsigned long*) &nextp, f ) ;
 
   /* save address of next directory entry, if any */
 
@@ -1145,6 +1197,12 @@ int fax_reset ( IFILE *f )
 int nextipage ( IFILE *f, int dp )
 {
   int err=0 ;
+  char *message ;
+
+#ifdef ENABLE_NLS
+  gsize written = 0 ;
+  char *conv_fname ;
+#endif
 
   int ( *reset [NPFORMATS] ) ( IFILE * ) = {
     raw_reset, fax_reset, pbm_reset, text_reset, pcx_reset
@@ -1171,8 +1229,25 @@ int nextipage ( IFILE *f, int dp )
 
   if ( ! err ) {
     f->f = fopen ( f->page->fname, (f->page->format == P_TEXT) ? "r" : "rb" ) ;
-    if ( ! f->f )
-      err = msg ( "ES2can't open %s:", f->page->fname ) ;
+    if ( ! f->f ) {
+      message = strdup2 ( "ES2 ", gettext ( "can't open file %s:" ) ) ;
+      if ( message ) {
+#ifdef ENABLE_NLS
+	if ( use_utf8 && !g_utf8_validate ( f->page->fname, -1, NULL ) ) {
+	  conv_fname = g_filename_to_utf8 ( f->page->fname, -1, NULL, &written, NULL ) ;
+	  if ( conv_fname ) {
+	    err = msg ( message, conv_fname ) ;
+	    g_free ( conv_fname ) ;
+	  }
+	} else {
+	  err = msg ( message, f->page->fname ) ;
+	}
+#else
+	err = msg ( message, f->page->fname ) ;
+#endif
+	free ( message ) ;
+      }
+    }
   }
 
   if ( ! err && fseek ( f->f, f->page->offset, SEEK_SET ) )
@@ -1223,6 +1298,12 @@ int newIFILE ( IFILE *f, char **fnames )
   char **p ;
   uchar buf[128] ;
   int ( *fun ) ( IFILE * ) ;
+  char *message ;
+
+#ifdef ENABLE_NLS
+  gsize written = 0 ;
+  char *conv_fname ;
+#endif
   
   int ( *first [NIFORMATS] ) ( IFILE * ) = {
     auto_first, pbm_first, fax_first, text_first, tiff_first, 
@@ -1240,24 +1321,91 @@ int newIFILE ( IFILE *f, char **fnames )
 
   for ( p=fnames ; ! err && *p ; p++ ) {
 
-    if ( ! ( f->f = fopen ( *p, "rb" ) ) )
-      err = msg ( "ES2 can't open %s:", *p ) ;
+    if ( ! ( f->f = fopen ( *p, "rb" ) ) ) {
+      message = strdup2 ( "ES2 ", gettext ( "can't open file %s:" ) ) ;
+      if ( message ) {
+#ifdef ENABLE_NLS
+	if ( use_utf8 && !g_utf8_validate ( *p, -1, NULL ) ) {
+	  conv_fname = g_filename_to_utf8 ( *p, -1, NULL, &written, NULL ) ;
+	  if ( conv_fname ) {
+	    err = msg ( message, conv_fname ) ;
+	    g_free ( conv_fname ) ;
+	  }
+	} else {
+	  err = msg ( message, *p ) ;
+	}
+#else
+	err = msg ( message, *p ) ;
+#endif
+	free ( message ) ;
+      }
+    }
     
     if ( ! err ) {
       n = fread ( buf, 1, 128, f->f ) ;
-      if ( ferror ( f->f ) )
-	err = msg ( "ES2 can't open %s:", *p ) ;
+      if ( ferror ( f->f ) ) {
+	message = strdup2 ( "ES2 ", gettext ( "can't open file %s:" ) ) ;
+	if ( message ) {
+#ifdef ENABLE_NLS
+	  if ( use_utf8 && !g_utf8_validate ( *p, -1, NULL ) ) {
+	    conv_fname = g_filename_to_utf8 ( *p, -1, NULL, &written, NULL ) ;
+	    if ( conv_fname ) {
+	      err = msg ( message, conv_fname ) ;
+	      g_free ( conv_fname ) ;
+	    }
+	  } else {
+	    err = msg ( message, *p ) ;
+	  }
+#else
+	  err = msg ( message, *p ) ;
+#endif
+	  free ( message ) ;
+	}
+      }
     }
-    
+      
     if ( ! err ) {
       fformat = getformat ( buf, n ) ;
-      if ( ! fformat )
-	err = msg ( "E2 can't get format of %s", *p ) ;
+      if ( ! fformat ) {
+	message = strdup2 ( "E2 ", gettext ( "can't get format of file %s" ) ) ;
+	if ( message ) {
+#ifdef ENABLE_NLS
+	  if ( use_utf8 && !g_utf8_validate ( *p, -1, NULL ) ) {
+	    conv_fname = g_filename_to_utf8 ( *p, -1, NULL, &written, NULL ) ;
+	    if ( conv_fname ) {
+	      err = msg ( message, conv_fname ) ;
+	      g_free ( conv_fname ) ;
+	    }
+	  } else {
+	    err = msg ( message, *p ) ;
+	  }
+#else
+	  err = msg ( message, *p ) ;
+#endif
+	  free ( message ) ;
+	}
+      }
     }
 
-    if ( ! err && fseek ( f->f, 0, SEEK_SET ) )
-      err = msg ( "ES2 can't rewind %s:", *p ) ;
-
+    if ( ! err && fseek ( f->f, 0, SEEK_SET ) ) {
+      message = strdup2 ( "ES2 ", gettext ( "can't rewind file %s:" ) ) ;
+      if ( message ) {
+#ifdef ENABLE_NLS
+	if ( use_utf8 && !g_utf8_validate ( *p, -1, NULL ) ) {
+	  conv_fname = g_filename_to_utf8 ( *p, -1, NULL, &written, NULL ) ;
+	  if ( conv_fname ) {
+	    err = msg ( message, conv_fname ) ;
+	    g_free ( conv_fname ) ;
+	  }
+	} else {
+	  err = msg ( message, *p ) ;
+	}
+#else
+	err = msg ( message, *p ) ;
+#endif
+	free ( message ) ;
+      }
+    }
     /* get format information for all pages in this file */
 
     for ( i=0 ; ! err ; i++ ) {
@@ -1768,7 +1916,13 @@ int nextopage ( OFILE *f, int page )
   int err = 0 ;
   int i, nb=0 ;
   uchar *p, codes [ ( RTCEOL * EOLBITS ) / 8 + 3 ] ;
+  char *message ;
   
+#ifdef ENABLE_NLS
+  gsize written = 0 ;
+  char *conv_cfname ;
+#endif
+
   if ( f->f ) { /* terminate previous page */
 
     switch ( f->format ) {
@@ -1836,7 +1990,23 @@ int nextopage ( OFILE *f, int page )
 	f->f = freopen ( f->cfname, ( f->format == O_PS ) ? "w" : "wb+", f->f ) ;
 
       if ( ! f->f ) {
-	err = msg ("ES2can't open output file %s:", f->cfname ) ;
+	message = strdup2 ( "ES2 ", gettext ( "can't open output file %s:" ) ) ;
+	if ( message ) {
+#ifdef ENABLE_NLS
+	  if ( use_utf8 && !g_utf8_validate ( f->cfname, -1, NULL ) ) {
+	    conv_cfname = g_filename_to_utf8 ( f->cfname, -1, NULL, &written, NULL ) ;
+	    if ( conv_cfname ) {
+	      err = msg ( message, conv_cfname ) ;
+	      g_free ( conv_cfname ) ;
+	    }
+	  } else {
+	    err = msg ( message, f->cfname ) ;
+	  }
+#else
+	  err = msg ( message, f->cfname ) ;
+#endif
+	  free ( message ) ;
+	}
       }
     } else {
       f->f = stdout ;
