@@ -39,6 +39,7 @@ COPYING distributed with the source files.
 #include "addressbook_icons.h"
 
 #ifdef HAVE_OSTREAM
+#include <ios>
 #include <ostream>
 #endif
 
@@ -127,7 +128,7 @@ AddressBook::AddressBook(const int size, GtkWindow* parent_p):
 				 GTK_POLICY_ALWAYS);
 
   // create the tree model and put it in a GobjHandle to handle its
-  // lifetime, because it is derived from GObject but not GtkObject
+  // lifetime, because it is not owned by a GTK+ container
   list_store_h =
     GobjHandle<GtkTreeModel>(GTK_TREE_MODEL(gtk_list_store_new(ModelColumns::cols_num,
 							       G_TYPE_STRING, G_TYPE_STRING)));
@@ -170,8 +171,7 @@ AddressBook::AddressBook(const int size, GtkWindow* parent_p):
   GtkWidget* image_p;
   { // provide a scope block for the GobjHandle
 
-    // GdkPixbufs are derived from GObject but not GtkObject
-    // so use a GobjHandle to handle their lifetime
+    // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
     GobjHandle<GdkPixbuf> pixbuf_h(gdk_pixbuf_new_from_xpm_data(add_xpm));
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(add_button_p), image_p);
@@ -565,12 +565,7 @@ gboolean AddressDialogCB::addr_dialog_key_press_event(GtkWidget*, GdkEventKey* e
   int keycode = event_p->keyval;
   bool cancel_focus = GTK_WIDGET_HAS_FOCUS(instance_p->cancel_button_p);
 
-  if (keycode == GDK_Escape) {
-    gtk_widget_hide_all(GTK_WIDGET(instance_p->get_win()));
-    instance_p->close();
-    return true;  // stop processing here
-  }
-  else if (keycode == GDK_Return && !cancel_focus) {
+  if (keycode == GDK_Return && !cancel_focus) {
     if (instance_p->selected_impl()) instance_p->close();
     return true;  // stop processing here
   }
