@@ -28,28 +28,24 @@ COPYING distributed with the source files.
 #include <gdk/gdkpixbuf.h>
 #include <gdk/gdkkeysyms.h> // the key codes are here
 
-// a guess for the future!
-#if GTK_MAJOR_VERSION > 2
-#undef EFAX_GTK_USE_FILE_SEL
-// a pick for the present
-#elif GTK_MINOR_VERSION >= 4
-#undef EFAX_GTK_USE_FILE_SEL
+#if GTK_CHECK_VERSION(2,4,0)
+#  undef EFAX_GTK_USE_FILE_SEL
 #else
-#define EFAX_GTK_USE_FILE_SEL 1
+#  define EFAX_GTK_USE_FILE_SEL 1
 #endif
 
 #ifdef EFAX_GTK_USE_FILE_SEL
-#include <gtk/gtkfilesel.h>
-#define EFAX_GTK_FILE_DIALOG_NEW gtk_file_selection_new(0)
+#  include <gtk/gtkfilesel.h>
+#  define EFAX_GTK_FILE_DIALOG_NEW gtk_file_selection_new(0)
 #else
-#include <gtk/gtkfilechooserdialog.h>
-#include <glib/glist.h>
-#define EFAX_GTK_FILE_DIALOG_NEW (gtk_file_chooser_dialog_new(0, 0, \
-                                 GTK_FILE_CHOOSER_ACTION_OPEN, \
-                                 gettext("View"), GTK_RESPONSE_NONE, \
-                                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, \
-                                 GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, \
-                                 static_cast<void*>(0)))
+#  include <gtk/gtkfilechooserdialog.h>
+#  include <glib/glist.h>
+#  define EFAX_GTK_FILE_DIALOG_NEW (gtk_file_chooser_dialog_new(0, 0, \
+                                   GTK_FILE_CHOOSER_ACTION_OPEN, \
+                                   gettext("View"), GTK_RESPONSE_NONE, \
+                                   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, \
+                                   GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, \
+                                   static_cast<void*>(0)))
 #endif
 
 #include <pango/pango-font.h>
@@ -365,11 +361,8 @@ void GplDialogCB::gpl_style_set(GtkWidget*, GtkStyle*, void* data) {
   //now set the buttons in GplDialog
   int width = 0;
   int height = 0;
-  // create a button to work on
+  // create a button to work on - the GobjHandle object will claim ownership of the button
   GobjHandle<GtkWidget> button_h(gtk_button_new());
-  // simulate its placement in a container
-  g_object_ref(G_OBJECT(button_h.get()));
-  gtk_object_sink(GTK_OBJECT(button_h.get()));
 
   GtkStyle* style_p = gtk_widget_get_style(GTK_WIDGET(instance_p->get_win()));
   gtk_widget_set_style(button_h, style_p);
@@ -417,7 +410,7 @@ GplDialog::GplDialog(int size): WinBase(gettext("efax-gtk: Conditions, Notices a
   pango_font_description_free(font_description);
 
   gtk_text_buffer_set_text(gtk_text_view_get_buffer(text_view_p),
-			   copyright_msg,
+			   gpl_copyright_msg()->c_str(),
 			   -1);
 
   GtkScrolledWindow* scrolled_window_p = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(0, 0));
@@ -488,11 +481,8 @@ void GplDialog::get_longest_button_text(void) {
   text_vec.push_back(gettext("Accept"));
   text_vec.push_back(gettext("Reject"));
 
-  // create a button to work on
+  // create a button to work on - the GobjHandle object will claim ownership of the button
   GobjHandle<GtkWidget> button_h(gtk_button_new());
-  // simulate its placement in a container
-  g_object_ref(G_OBJECT(button_h.get()));
-  gtk_object_sink(GTK_OBJECT(button_h.get()));
 
   std::vector<std::string>::const_iterator temp_iter;
   std::vector<std::string>::const_iterator max_width_iter;
@@ -522,14 +512,6 @@ void InfoDialogCB::info_selected(GtkDialog*, int, void* data) {
   static_cast<InfoDialog*>(data)->close();
 }
 
-gboolean InfoDialogCB::info_key_press_event(GtkWidget*, GdkEventKey* event_p, void* data) {
-  if (event_p->keyval == GDK_Return || event_p->keyval == GDK_Escape) {
-    static_cast<InfoDialog*>(data)->close();
-    return true; // finish processing here
-  }
-  return false;  // continue processing key events
-}
-
 } // anonymous namespace
 
 InfoDialog::InfoDialog(const char* text, const char* caption,
@@ -552,8 +534,6 @@ InfoDialog::InfoDialog(const char* text, const char* caption,
 
   g_signal_connect(G_OBJECT(get_win()), "response",
 		   G_CALLBACK(InfoDialogCB::info_selected), this);
-  g_signal_connect(G_OBJECT(get_win()), "key_press_event",
-		   G_CALLBACK(InfoDialogCB::info_key_press_event), this);
 
   gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER_ON_PARENT);
   gtk_window_set_resizable(get_win(), false);
