@@ -6,6 +6,14 @@ COPYING distributed with the source files.
 
 */
 
+// something in gtkprintjob.h clashes with the C++ headers with
+// libstdc++-5 and it needs to be included here to obtain a
+// clean compile
+#include <gtk/gtkversion.h>
+#if GTK_CHECK_VERSION(2,10,0)
+#include <gtk/gtkprintjob.h>
+#endif
+
 #include <vector>
 #include <iomanip>
 #include <memory>
@@ -50,13 +58,12 @@ COPYING distributed with the source files.
 #include "fax_list.h"
 #include "dialogs.h"
 #include "fax_list_icons.h"
-#include "utils/pipes.h"
 #include "utils/thread.h"
-#include "utils/mutex.h"
 #include "utils/shared_handle.h"
 #include "utils/utf8_utils.h"
 
 #ifdef HAVE_OSTREAM
+#include <ios>
 #include <ostream>
 #endif
 
@@ -66,6 +73,10 @@ COPYING distributed with the source files.
 #include <strstream>
 #endif
 
+#ifdef HAVE_STREAM_IMBUE
+#include <locale>
+#endif
+
 #ifdef ENABLE_NLS
 #include <libintl.h>
 #endif
@@ -117,9 +128,16 @@ void FaxListDialogCB::fax_list_button_clicked(GtkWidget* widget_p, void* data) {
 FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
 			      WinBase(0, prog_config.window_icon_h, false),
                               mode(mode_), standard_size(standard_size_),
-			      fax_basename_sem(1),
+			      working_dir(prog_config.working_dir),
                               fax_list_manager(mode) {
 
+  // we have included a std::string working_dir member so that we can access it in the
+  // FaxListDialog::print_fax_thread() and FaxListDialog::view_fax_thread() without
+  // having to introduce locking of accesses to prog_config.working_dir by the main
+  // GUI thread.  It is fine to initialise it in the initialisation list of this
+  // constructor as prog_config.working_dir is only set once, on the first call to
+  // configure_prog() (that is, when configure_prog() is passed false as its argument)
+
   // notify the existence of this object
   if (mode == FaxListEnum::received) is_fax_received_list++;
   else is_fax_sent_list++;
@@ -176,6 +194,7 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
   gtk_button_set_relief(GTK_BUTTON(print_button_p), GTK_RELIEF_NONE);
 
   { // scope block for the GobjHandle
+    // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
     GobjHandle<GdkPixbuf> pixbuf_h(gdk_pixbuf_new_from_xpm_data(view_xpm));
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(view_button_p), image_p);
@@ -186,6 +205,7 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
   gtk_button_set_relief(GTK_BUTTON(view_button_p), GTK_RELIEF_NONE);
 
   { // scope block for the GobjHandle
+    // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
     GobjHandle<GdkPixbuf> pixbuf_h(gdk_pixbuf_new_from_xpm_data(describe_xpm));
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(describe_button_p), image_p);
@@ -210,6 +230,7 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
   gtk_button_set_relief(GTK_BUTTON(empty_trash_button_p), GTK_RELIEF_NONE);
 
   { // scope block for the GobjHandle
+    // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
     GobjHandle<GdkPixbuf> pixbuf_h(gdk_pixbuf_new_from_xpm_data(add_folder_xpm));
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(add_folder_button_p), image_p);
@@ -220,6 +241,7 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
   gtk_button_set_relief(GTK_BUTTON(add_folder_button_p), GTK_RELIEF_NORMAL);
 
   { // scope block for the GobjHandle
+    // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
     GobjHandle<GdkPixbuf> pixbuf_h(gdk_pixbuf_new_from_xpm_data(delete_folder_xpm));
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(delete_folder_button_p), image_p);
@@ -520,8 +542,12 @@ void FaxListDialog::print_fax_prompt(void) {
     // bool which can be safely concurrently read in different threads and in any
     // event it is read in FaxListDialog::print_fax_thread() in a separate process
     // after fork()ing so there isn't even concurrent access
+#if GTK_CHECK_VERSION(2,10,0)
+    if (prog_config.gtkprint) print_fax();
+    else if (prog_config.print_popup) {
+#else
     if (prog_config.print_popup) {
-      
+#endif      
       std::string msg(gettext("Print selected fax?"));
       PromptDialog* dialog_p = new PromptDialog(msg.c_str(), gettext("efax-gtk: Print fax"),
 						standard_size, get_win());
@@ -624,6 +650,11 @@ std::pair<const char*, char* const*> FaxListDialog::get_fax_to_ps_parms
       efix_parms.push_back(temp);
 
       std::ostringstream strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+      strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
       if (!prog_config.page_size.compare("a4")) {
 
 	strm << "-d";
@@ -660,6 +691,11 @@ std::pair<const char*, char* const*> FaxListDialog::get_fax_to_ps_parms
 
     int partnumber = 1;
     std::ostringstream strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+    strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
     strm << basename.c_str() << std::setfill('0')
 	 << std::setw(3) << partnumber;
     int result = access(strm.str().c_str(), R_OK);
@@ -681,6 +717,11 @@ std::pair<const char*, char* const*> FaxListDialog::get_fax_to_ps_parms
       efix_parms.push_back(temp);
 
       std::ostrstream strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+      strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
       if (!prog_config.page_size.compare("a4")) {
 
 	strm << "-d";
@@ -723,6 +764,11 @@ std::pair<const char*, char* const*> FaxListDialog::get_fax_to_ps_parms
 
     int partnumber = 1;
     std::ostrstream strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+    strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
     strm << basename.c_str() << std::setfill('0')
 	 << std::setw(3) << partnumber << std::ends;
     const char* file_name = strm.str();
@@ -734,6 +780,11 @@ std::pair<const char*, char* const*> FaxListDialog::get_fax_to_ps_parms
 
       partnumber++;
       std::ostrstream strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+      strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
       strm << basename.c_str() << std::setfill('0')
 	   << std::setw(3) << partnumber << std::ends;
       file_name = strm.str();
@@ -760,25 +811,22 @@ std::pair<const char*, char* const*> FaxListDialog::get_fax_to_ps_parms
   return std::pair<const char*, char* const*>(prog_name, exec_parms);
 }
 
-void FaxListDialog::write_from_stdin(int dest_fd) {
+void FaxListDialog::write_pipe_to_file(PipeFifo* pipe_p, int file_fd) {
 
   ssize_t read_result;
   ssize_t write_result;
   ssize_t written;
   char buffer[PIPE_BUF];
 
-  while ((read_result = read(0, buffer, PIPE_BUF)) > 0
-	 || read_result == -1 && errno == EINTR) {
-    if (read_result > 0) {
-      written = 0;
-      do {
-	write_result = write(dest_fd, buffer + written, read_result);
-	if (write_result > 0) {
-	  written += write_result;
-	  read_result -= write_result;
-	}
-      } while (read_result && (write_result != -1 || errno == EINTR));
-    }
+  while ((read_result = pipe_p->read(buffer, PIPE_BUF)) > 0) {
+    written = 0;
+    do {
+      write_result = write(file_fd, buffer + written, read_result);
+      if (write_result > 0) {
+	written += write_result;
+	read_result -= write_result;
+      }
+    } while (read_result && (write_result != -1 || errno == EINTR));
   }
   g_usleep(50000);
 }
@@ -850,14 +898,10 @@ std::pair<const char*, char* const*> FaxListDialog::get_ps_viewer_parms(const ch
 void FaxListDialog::print_fax(void) {
 
   if (fax_list_manager.is_fax_selected() == 1 && !prog_config.print_cmd.empty()) {
-    // fax_basename_sem is initialised to a count of 1, so this will stop
-    // fax_basename being overwritten until SemSync::post() is called in
-    // print_fax_thread() or view_fax_thread() or on a thread creation error
-    fax_basename_sem.wait();
 
-    fax_basename = prog_config.working_dir;
-    if (mode == FaxListEnum::received) fax_basename += "/faxin/";
-    else fax_basename += "/faxsent/";
+    std::auto_ptr<std::string> fax_basename_a(new std::string(prog_config.working_dir));
+    if (mode == FaxListEnum::received) *fax_basename_a += "/faxin/";
+    else *fax_basename_a += "/faxsent/";
 
     // we don't need to use a Glib conversion function here - we know the
     // fax name is just plain ASCII numbers
@@ -866,10 +910,15 @@ void FaxListDialog::print_fax(void) {
     // as we checked for a fax selection, fax_number_h should not be
     // null, but check it in case
     if (fax_number_h.get()) fax_number = fax_number_h.get();
-    fax_basename += fax_number;
-    fax_basename += '/';
-    fax_basename += fax_number;
-    fax_basename += '.';
+    *fax_basename_a += fax_number;
+    *fax_basename_a += '/';
+    *fax_basename_a += fax_number;
+    *fax_basename_a += '.';
+
+    // create the print manager
+#if GTK_CHECK_VERSION(2,10,0)
+    if (prog_config.gtkprint) async_queue.push(PrintManager::create_manager(get_win()));
+#endif
 
     // now block off the signals for which we have set handlers so that the worker
     // thread does not receive the signals, otherwise we will have memory synchronisation
@@ -884,29 +933,127 @@ void FaxListDialog::print_fax(void) {
     sigaddset(&sig_mask, SIGHUP);
     pthread_sigmask(SIG_BLOCK, &sig_mask, 0);
 
+    // hand ownership to the new thread - Thread::Thread::start() could throw
+    // in the event of memory allocation failure but as a matter of policy we
+    // do not deal with that condition in this program so we can pass a raw pointer
+    // rather than the std::auto_ptr<> object - anyway passing a std::auto_ptr<>
+    // object wouldn't be enough as we would still have to catch the std::bad_alloc
+    // exception in order to pop() off the object in async_queue
+    std::string* arg_p = fax_basename_a.release();
     std::auto_ptr<Thread::Thread> res =
-      Thread::Thread::start(sigc::mem_fun(*this, &FaxListDialog::print_fax_thread), false);
+      Thread::Thread::start(sigc::bind(sigc::mem_fun(*this, &FaxListDialog::print_fax_thread),
+				       arg_p, prog_config.gtkprint), false);
     if (!res.get()) {
+      delete arg_p;
       write_error("Cannot start new print thread, fax will not be printed\n");
-      fax_basename_sem.post();
+#if GTK_CHECK_VERSION(2,10,0)
+      // Because another thread (for another print job) could have been
+      // launched this may extract a different print manager object from
+      // the one created above in this thread, but that doesn't matter as
+      // they are all the same - the filename passed as the argument to
+      // print_fax_thread() which in fact determines the print job printed
+      if (prog_config.gtkprint) {
+	try {
+	  async_queue.pop();
+	}
+	catch (AsyncQueuePopError& error) {
+	  write_error(error.what());
+	  write_error(" in FaxListDialog::print_fax()\n");
+	}
+      }
+#endif
     }
     // now unblock the signals so that the initial (GUI) thread can receive them
     pthread_sigmask(SIG_UNBLOCK, &sig_mask, 0);
   }
 }
 
-void FaxListDialog::print_fax_thread(void) {
+void FaxListDialog::print_fax_thread(std::string* arg_p, bool use_gtkprint) {
 
+  // extract the string argument to a std::auto_ptr<> object
+  const std::auto_ptr<std::string> fax_basename_a(arg_p);
+
+  // extract any PrintManager object
+#if GTK_CHECK_VERSION(2,10,0)
+  IntrusivePtr<PrintManager> print_manager_i;
+  if (use_gtkprint) {
+    try {
+     async_queue.pop(print_manager_i);
+    }
+    catch (AsyncQueuePopError& error) {
+      write_error(error.what());
+      write_error(" in FaxListDialog::print_fax_thread()\n");
+      return;
+    }
+  }
+#endif
   // get the arguments for the exec() calls below (because this is a
   // multi- threaded program, we must do this before fork()ing because
   // we use functions to get the arguments which are not async-signal-safe)
+  std::pair<const char*, char* const*> fax_to_ps_parms(get_fax_to_ps_parms(*fax_basename_a, true));
+
+#if GTK_CHECK_VERSION(2,10,0)
+  if (use_gtkprint) {
+
+    // get a temporary file
+    std::string filename;
+    {
+      Thread::Mutex::Lock lock(working_dir_mutex);
+      filename = working_dir;
+    }
+    filename += "/efax-gtk-print-XXXXXX";
+
+    std::string::size_type size = filename.size() + 1;
+    ScopedHandle<char*> tempfile_h(new char[size]);
+    std::memcpy(tempfile_h.get(), filename.c_str(), size); // this will include the terminating '\0' in the copy
+    int file_fd = mkstemp(tempfile_h.get());
+
+    if (file_fd == -1) {
+      write_error("Failed to make temporary file for printing");
+      return;
+    }
+
+    // now create a pipe and proceed to fork to write the postscript to temporary file
+    PipeFifo fork_pipe(PipeFifo::block);
+
+    // now fork to create the process which will write postscript to stdout
+    pid_t pid = fork();
+
+    if (pid == -1) {
+      write_error("Fork error - exiting\n");
+      std::exit(FORK_ERROR);
+    }
+  
+    if (!pid) { // child process which will send postscript to stdout
+  
+      // this is the child process to write to stdin
+      // now we have fork()ed we can connect to stderr
+      connect_to_stderr();
+
+      fork_pipe.connect_to_stdout();
+      execvp(fax_to_ps_parms.first, fax_to_ps_parms.second);
+
+      // if we reached this point, then the execvp() call must have failed
+      // report error and then end process - use _exit(), not exit()
+      write_error("Can't find the efix-0.9a program - please check your installation\n"
+		  "and the PATH environmental variable\n");
+      _exit(0);
+    }
+
+    // write from the pipe (containing the generated PS text) to file
+    fork_pipe.make_readonly();
+    write_pipe_to_file(&fork_pipe, file_fd);
+    while (::close(file_fd) == -1 && errno == EINTR);
+
+    print_manager_i->set_filename(tempfile_h.get());
+    print_manager_i->print(); // this will pass ownership to the PrintManager object
+                              // which will control its own destiny now, via the GTK+
+                              // print system
+  }
+  else {
+#endif
+
   std::pair<const char*, char* const*> print_from_stdin_parms(get_print_from_stdin_parms());
-  std::pair<const char*, char* const*> fax_to_ps_parms(get_fax_to_ps_parms(fax_basename, true));
-
-  // release main GUI thread in print_fax() or view_fax()
-  // waiting for this use of fax_basename to finish
-  fax_basename_sem.post();
-
   if (print_from_stdin_parms.first) { // this will be 0 if get_print_from_stdin_parms()
                                       // threw a Utf8::ConversionError)
 
@@ -931,7 +1078,7 @@ void FaxListDialog::print_fax_thread(void) {
       if (prog_config.print_popup) connect_to_stderr();
 
       // now create a blocking pipe which the print processes can use to
-      // communicate via stdout/stdin
+      // communicate
       PipeFifo fork_pipe;
       try {
 	fork_pipe.open(PipeFifo::block);
@@ -974,6 +1121,9 @@ void FaxListDialog::print_fax_thread(void) {
     // only being able to use async-signal-safe functions
     delete_parms(print_from_stdin_parms);
   }
+#if GTK_CHECK_VERSION(2,10,0)
+  }
+#endif
   delete_parms(fax_to_ps_parms);
 }
 
@@ -986,14 +1136,10 @@ void FaxListDialog::view_fax(void) {
   // FaxListsDialog::view_fax_thread() and is therefore not running
   // at this stage
   if (fax_list_manager.is_fax_selected() == 1 && !prog_config.ps_view_cmd.empty()) {
-    // fax_basename_sem is initialised to a count of 1, so this will stop
-    // fax_basename being overwritten until SemSync::post() is called in
-    // print_fax_thread() or view_fax_thread() or on a thread creation error
-    fax_basename_sem.wait();
 
-    fax_basename = prog_config.working_dir;
-    if (mode == FaxListEnum::received) fax_basename += "/faxin/";
-    else fax_basename += "/faxsent/";
+    std::auto_ptr<std::string> fax_basename_a(new std::string(prog_config.working_dir));
+    if (mode == FaxListEnum::received) *fax_basename_a += "/faxin/";
+    else *fax_basename_a += "/faxsent/";
 
     // we don't need to use a Glib conversion function here - we know the
     // fax name is just plain ASCII numbers
@@ -1002,10 +1148,10 @@ void FaxListDialog::view_fax(void) {
     // as we checked for a fax selection, fax_number_h should not be
     // null, but check it in case
     if (fax_number_h.get()) fax_number = fax_number_h.get();
-    fax_basename += fax_number;
-    fax_basename += '/';
-    fax_basename += fax_number;
-    fax_basename += '.';
+    *fax_basename_a += fax_number;
+    *fax_basename_a += '/';
+    *fax_basename_a += fax_number;
+    *fax_basename_a += '.';
 
     // now block off the signals for which we have set handlers so that the socket server
     // thread does not receive the signals, otherwise we will have memory synchronisation
@@ -1020,26 +1166,42 @@ void FaxListDialog::view_fax(void) {
     sigaddset(&sig_mask, SIGHUP);
     pthread_sigmask(SIG_BLOCK, &sig_mask, 0);
 
+    // hand ownership to the new thread - Thread::Thread::start() could throw
+    // in the event of memory allocation failure but as a matter of policy we
+    // do not deal with that condition in this program so we can pass a raw pointer
+    // rather than the std::auto_ptr<> object
+    std::string* arg_p = fax_basename_a.release();
     std::auto_ptr<Thread::Thread> res =
-      Thread::Thread::start(sigc::mem_fun(*this, &FaxListDialog::view_fax_thread), false);
+      Thread::Thread::start(sigc::bind(sigc::mem_fun(*this, &FaxListDialog::view_fax_thread), arg_p),
+			    false);
     if (!res.get()) {
+      delete arg_p;
       write_error("Cannot start new view fax thread, fax cannot be viewed\n");
-      fax_basename_sem.post();
     }
     // now unblock the signals so that the initial (GUI) thread can receive them
     pthread_sigmask(SIG_UNBLOCK, &sig_mask, 0);
   }
 }
 
-void FaxListDialog::view_fax_thread(void) {
+void FaxListDialog::view_fax_thread(std::string* arg_p) {
  
+  const std::auto_ptr<std::string> fax_basename_a(arg_p);
+
   // get a temporary file
-  char filename[] = "/tmp/efax-gtk-view.XXXXXX";
-  int file_fd = mkstemp(filename);
+  std::string filename;
+  {
+    Thread::Mutex::Lock lock(working_dir_mutex);
+    filename = working_dir;
+  }
+  filename += "/efax-gtk-view-XXXXXX";
+
+  std::string::size_type size = filename.size() + 1;
+  ScopedHandle<char*> tempfile_h(new char[size]);
+  std::memcpy(tempfile_h.get(), filename.c_str(), size); // this will include the terminating '\0' in the copy
+  int file_fd = mkstemp(tempfile_h.get());
 
   if (file_fd == -1) {
-    write_error("Failed to make temporary file:\n"
-		"please check permissions in /tmp directory");
+    write_error("Failed to make temporary file for viewing");
     return;
   }
 
@@ -1050,11 +1212,7 @@ void FaxListDialog::view_fax_thread(void) {
   // PS format following the next fork() call (because this is a multi-
   // threaded program, we must do this before fork()ing because we use
   // functions to get the arguments which are not async-signal-safe)
-  std::pair<const char*, char* const*> fax_to_ps_parms(get_fax_to_ps_parms(fax_basename, false));
-
-  // release main GUI thread in print_fax() or view_fax()
-  // waiting for this use of fax_basename to finish
-  fax_basename_sem.post();
+  std::pair<const char*, char* const*> fax_to_ps_parms(get_fax_to_ps_parms(*fax_basename_a, false));
 
   // now fork to create the process which will write postscript to stdout
   pid_t pid = fork();
@@ -1080,7 +1238,7 @@ void FaxListDialog::view_fax_thread(void) {
     _exit(0);
   }
   // this is the parent process, which will convert the fax to Postscript
-  // and then pipe it on stdout to write_from_stdin()
+  // and then pipe it on stdout to write_to_file()
     
   // first, release the memory allocated on the heap for
   // the redundant fax_to_ps_parms
@@ -1088,9 +1246,10 @@ void FaxListDialog::view_fax_thread(void) {
   // only being able to use async-signal-safe functions
   delete_parms(fax_to_ps_parms);
     
-  // now write stdin in (containing the generated PS text) to file
-  fork_pipe.connect_to_stdin();
-  write_from_stdin(file_fd);
+  // now write from the pipe (containing the generated PS text) to file
+  fork_pipe.make_readonly();
+  write_pipe_to_file(&fork_pipe, file_fd);
+  while (::close(file_fd) == -1 && errno == EINTR);
 
   // the temporary file to be viewed has now been created
   // so launch a new process to control the viewing process
@@ -1101,7 +1260,7 @@ void FaxListDialog::view_fax_thread(void) {
   // viewer following the next fork() call (because this is a multi-
   // threaded program, we must do this before fork()ing because we use
   // functions to get the arguments which are not async-signal-safe)
-  std::pair<const char*, char* const*> ps_viewer_parms(get_ps_viewer_parms(filename));
+  std::pair<const char*, char* const*> ps_viewer_parms(get_ps_viewer_parms(tempfile_h.get()));
 
   if (ps_viewer_parms.first) { // this will be 0 if get_ps_viewer_parms()
                                // threw a Utf8::ConversionError)
@@ -1161,7 +1320,7 @@ void FaxListDialog::view_fax_thread(void) {
       // call to wait()
       wait(0);
     
-      unlink(filename);
+      unlink(tempfile_h.get());
       // now terminate the process
       _exit(0);
     }
@@ -1240,12 +1399,7 @@ gboolean EntryDialogCB::entry_dialog_key_press_event(GtkWidget*, GdkEventKey* ev
   int keycode = event_p->keyval;
   bool cancel_focus = GTK_WIDGET_HAS_FOCUS(instance_p->cancel_button_p);
 
-  if (keycode == GDK_Escape) {
-    gtk_widget_hide_all(GTK_WIDGET(instance_p->get_win()));
-    instance_p->close();
-    return true;  // stop processing here
-  }
-  else if (keycode == GDK_Return && !cancel_focus) {
+  if (keycode == GDK_Return && !cancel_focus) {
     if (instance_p->selected_impl()) instance_p->close();
     return true;  // stop processing here
   }
