@@ -11,6 +11,16 @@ COPYING distributed with the source files.
 
 #include "prog_defs.h"
 
+// something in gtkprintjob.h clashes with the C++ headers with
+// libstdc++-5 and it needs to be included here (via print_manager.h)
+// to obtain a clean compile
+#include <gtk/gtkversion.h>
+#if GTK_CHECK_VERSION(2,10,0)
+#include "print_manager.h"
+#include "utils/intrusive_ptr.h"
+#include "utils/async_queue.h"
+#endif
+
 #include <string>
 #include <utility>
 
@@ -21,7 +31,8 @@ COPYING distributed with the source files.
 
 #include "fax_list_manager.h"
 #include "utils/window.h"
-#include "utils/sem_sync.h"
+#include "utils/pipes.h"
+#include "utils/mutex.h"
 
 namespace { // we put the function in anonymous namespace in fax_list.cpp
             // so it is not exported at link time
@@ -36,11 +47,15 @@ class FaxListDialog: public sigc::trackable, public WinBase {
   static int is_fax_sent_list;
   const int standard_size;
 
-  SemSync fax_basename_sem;
-  std::string fax_basename;
+  Thread::Mutex working_dir_mutex;
+  std::string working_dir;
 
   FaxListManager fax_list_manager;
 
+#if GTK_CHECK_VERSION(2,10,0)
+  AsyncQueue<IntrusivePtr<PrintManager> > async_queue;
+#endif
+
   GtkWidget* close_button_p;
   GtkWidget* print_button_p;
   GtkWidget* view_button_p;
@@ -60,15 +75,15 @@ class FaxListDialog: public sigc::trackable, public WinBase {
   void add_folder(const std::string&);
   void empty_trash_prompt(void);
   void delete_folder_prompt(void);
-  void write_from_stdin(int);
+  void write_pipe_to_file(PipeFifo*, int);
   std::pair<const char*, char* const*> get_print_from_stdin_parms(void);
   std::pair<const char*, char* const*> get_fax_to_ps_parms(const std::string&, bool);
   std::pair<const char*, char* const*> get_ps_viewer_parms(const char*);
   void print_fax_prompt(void);
   void print_fax(void);
-  void print_fax_thread(void);
+  void print_fax_thread(std::string*, bool);
   void view_fax(void);
-  void view_fax_thread(void);
+  void view_fax_thread(std::string*);
   void delete_parms(std::pair<const char*, char* const*>);
 public:
   friend void FaxListDialogCB::fax_list_button_clicked(GtkWidget*, void*);
