@@ -1,13 +1,16 @@
 #ifndef GPL_H
 #define GPL_H
 
-const char copyright_msg[] = 
+#include <string>
+#include <memory>
+
+const char copyright_msg_part_1[] = 
 
 "\n\n"
 
 "-------------------------------------------------------------------------\n"
 "\n"
-"Copyright (C) 2001 to 2005 Chris Vine\n"
+"Copyright (C) 2001 to 2006 Chris Vine\n"
 "\n"
 "\n"
 "    This program is free software; you can redistribute it and/or modify\n"
@@ -25,15 +28,19 @@ const char copyright_msg[] =
 "    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n"
 "    or from the COPYING file with this distribution\n"
 "\n"
-"    Note that the licence below is copyrighted by the Free Software\n"
-"    Foundation, but the instance of code that it refers to (this program)\n"
-"    is copyrighted by me.  The efax program, to which this is a front\n"
-"    end, is copyrighted by Ed Casas.\n"
+"    Note that the copyright to the licence below is held by the Free\n"
+"    Software Foundation, but the copyright to the instance of code that\n"
+"    it refers to (this program) is held by me.  The copyright to the efax\n"
+"    program, to which this is a front end, is held by Ed Casas.\n"
 "\n"
 "Chris Vine\n"
 "\n"
 "-------------------------------------------------------------------------\n"
 "\n"
+;
+
+const char copyright_msg_part_2[] = 
+
 "		    GNU GENERAL PUBLIC LICENSE\n"
 "		       Version 2, June 1991\n"
 "\n"
@@ -84,6 +91,10 @@ const char copyright_msg[] =
 "that any problems introduced by others will not reflect on the original\n"
 "authors' reputations.\n"
 "\n"
+;
+
+const char copyright_msg_part_3[] = 
+
 "  Finally, any free program is threatened constantly by software\n"
 "patents.  We wish to avoid the danger that redistributors of a free\n"
 "program will individually obtain patent licenses, in effect making the\n"
@@ -138,6 +149,10 @@ const char copyright_msg[] =
 "    part thereof, to be licensed as a whole at no charge to all third\n"
 "    parties under the terms of this License.\n"
 "\n"
+;
+
+const char copyright_msg_part_4[] = 
+
 "    c) If the modified program normally reads commands interactively\n"
 "    when run, you must cause it, when started running for such\n"
 "    interactive use in the most ordinary way, to print or display an\n"
@@ -190,6 +205,10 @@ const char copyright_msg[] =
 "    received the program in object code or executable form with such\n"
 "    an offer, in accord with Subsection b above.)\n"
 "\n"
+;
+
+const char copyright_msg_part_5[] = 
+
 "The source code for a work means the preferred form of the work for\n"
 "making modifications to it.  For an executable work, complete source\n"
 "code means all the source code for all modules it contains, plus any\n"
@@ -245,6 +264,10 @@ const char copyright_msg[] =
 "the only way you could satisfy both it and this License would be to\n"
 "refrain entirely from distribution of the Program.\n"
 "\n"
+;
+
+const char copyright_msg_part_6[] = 
+
 "If any portion of this section is held invalid or unenforceable under\n"
 "any particular circumstance, the balance of the section is intended to\n"
 "apply and the section as a whole is intended to apply in other\n"
@@ -293,6 +316,10 @@ const char copyright_msg[] =
 "of preserving the free status of all derivatives of our free software and\n"
 "of promoting the sharing and reuse of software generally.\n"
 "\n"
+;
+
+const char copyright_msg_part_7[] = 
+
 "			    NO WARRANTY\n"
 "\n"
 "  11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY\n"
@@ -318,4 +345,17 @@ const char copyright_msg[] =
 "		     END OF TERMS AND CONDITIONS\n"
 ;
 
+inline std::auto_ptr<std::string> gpl_copyright_msg(void) {
+
+  std::auto_ptr<std::string> return_val(new std::string(copyright_msg_part_1));
+  return_val->append(copyright_msg_part_2);
+  return_val->append(copyright_msg_part_3);
+  return_val->append(copyright_msg_part_4);
+  return_val->append(copyright_msg_part_5);
+  return_val->append(copyright_msg_part_6);
+  return_val->append(copyright_msg_part_7);
+
+  return return_val;
+}
+
 #endif
