@@ -8,6 +8,14 @@ COPYING distributed with the source files.
 
 #include "prog_defs.h"
 
+// something in gtkprintjob.h clashes with the C++ headers with
+// libstdc++-5 and it needs to be included here to obtain a
+// clean compile
+#include <gtk/gtkversion.h>
+#if GTK_CHECK_VERSION(2,10,0)
+#include <gtk/gtkprintjob.h>
+#endif
+
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -23,19 +31,25 @@ COPYING distributed with the source files.
 #include <string>
 #include <vector>
 #include <algorithm>
+#include <exception>
 #include <cstring>
 #include <ctime>
 
 #ifdef HAVE_OSTREAM
+#include <ios>
 #include <ostream>
 #endif
 
 #ifdef HAVE_ISTREAM
+#include <ios>
 #include <istream>
 #endif
 
+#ifdef HAVE_STREAM_IMBUE
+#include <locale>
+#endif
+
 #include <gtk/gtkmain.h>
-#include <gtk/gtkversion.h>
 
 #include <gdk/gdkpixbuf.h>
 #include <gdk/gdk.h>   // for gdk_notify_startup_complete() and gdk_beep()
@@ -48,24 +62,16 @@ COPYING distributed with the source files.
 #include <gdk/gdkx.h>  // for GDK_DISPLAY()
 #endif
 
-// a guess for the future!
-#if GTK_MAJOR_VERSION > 2
-#define EFAX_GTK_USE_ICON_THEME 1
-// a pick for the present
-#elif GTK_MINOR_VERSION >= 4
-#define EFAX_GTK_USE_ICON_THEME 1
+#if GTK_CHECK_VERSION(2,4,0)
+#  define EFAX_GTK_USE_ICON_THEME 1
 #else
-#undef EFAX_GTK_USE_ICON_THEME
+#  undef EFAX_GTK_USE_ICON_THEME
 #endif
 
-// a guess for the future!
-#if GTK_MAJOR_VERSION > 2
-#define HAVE_NOTIFY_STARTUP 1
-// a pick for the present
-#elif GTK_MINOR_VERSION >= 2
-#define HAVE_NOTIFY_STARTUP 1
+#if GTK_CHECK_VERSION(2,2,0)
+#  define HAVE_NOTIFY_STARTUP 1
 #else
-#undef HAVE_NOTIFY_STARTUP
+#  undef HAVE_NOTIFY_STARTUP
 #endif
 
 #ifdef EFAX_GTK_USE_ICON_THEME
@@ -125,20 +131,32 @@ int main(int argc, char* argv[]) {
   bind_textdomain_codeset("efax-gtk", "UTF-8");
   textdomain("efax-gtk");
   setlocale(LC_ALL,"");
+#  ifdef HAVE_STREAM_IMBUE
+  // make sure that the locale is set for C++ objects also
+  // (this should also set it for C functions but calling setlocale()
+  // explicitly above will also ensure POSIX compliance)
+  try {
+    std::locale::global(std::locale(""));
+  }
+  catch (std::exception& except) {
+    const std::string message(except.what());
+    write(2, message.data(), message.size());
+  }
+#  endif
 #endif
 
   // provide a new process group with this process as the process group leader
   // so we can check whether another process is one of our children (ie a member
   // of our process group) and also call kill(0, SIGTERM) in atexit_cleanup()
   if (setpgid(getpid(), getpid()) == -1) {
-    std::string message("Error in creating new process group\n");
+    const std::string message("Error in creating new process group\n");
     write(2, message.data(), message.size());
   }
 
   if (argc > 1) {
     if (is_arg("--version", argc, argv)) {
       std::string message("efax-gtk-" VERSION "\n");
-      message += gettext("Copyright (C) 2001 - 2005 Chris Vine\n"
+      message += gettext("Copyright (C) 2001 - 2006 Chris Vine\n"
 			 "This program is released under the "
 			 "GNU General Public License, version 2\n");
       try {
@@ -382,6 +400,7 @@ std::string configure_prog(bool reread) {
       prog_config.sock_server_port = "";
       prog_config.fax_received_prog = "";
       prog_config.logfile_name = "";
+      prog_config.permitted_clients_list.clear();
     }
     // if we are setting up for first time, initialise prog_config.efax_controller_child_pid;
     else prog_config.efax_controller_child_pid = 0;
@@ -398,6 +417,7 @@ std::string configure_prog(bool reread) {
     std::string reset;
     std::string capabilities;
     std::string extra_parms;
+    std::string gtkprint;
     std::string print_popup;
     std::string sock_server;
     std::string sock_popup;
@@ -458,6 +478,9 @@ std::string configure_prog(bool reread) {
 	// look for "PRINT_CMD:"
 	else if (get_prog_parm("PRINT_CMD:", file_read, prog_config.print_cmd));
 
+	// look for "USE_GTKPRINT:"
+	else if (get_prog_parm("USE_GTKPRINT:", file_read, gtkprint));
+
 	// look for "PRINT_SHRINK:"
 	else if (get_prog_parm("PRINT_SHRINK:", file_read, prog_config.print_shrink));
 
@@ -725,10 +748,16 @@ std::string configure_prog(bool reread) {
       }
     }
     if (prog_config.print_cmd.empty()) {
-      return_val += gettext("Adopting default printer command of 'lpr'\n");
       prog_config.print_cmd = "lpr";
     }
     
+    temp = Utf8::to_lower(gtkprint);
+#if !(GTK_CHECK_VERSION(2,10,0))
+    temp = "no";
+#endif
+    if (!temp.compare("yes")) prog_config.gtkprint = true;
+    else prog_config.gtkprint = false;
+
     if (prog_config.print_shrink.empty()) {
       prog_config.print_shrink = "100";
     }
@@ -1000,6 +1029,11 @@ bool is_program_running(void) {
 	unlink(lockfile.c_str());
       }
       else { // lock file already exists and we can open it - check if stale
+
+#ifdef HAVE_STREAM_IMBUE
+	existing_lock.imbue(std::locale::classic());
+#endif // HAVE_STREAM_IMBUE
+
 	pid_t existing_lock_pid;
 	existing_lock >> existing_lock_pid;
 	if (existing_lock && (kill(existing_lock_pid, 0)) < 0) { // stale lock
@@ -1021,6 +1055,11 @@ bool is_program_running(void) {
 	    if (write_fd >= 0) {                  // if we don't enter this block we also have a
                                                   // problem - the fifo can't be opened for writing
 	      Attachable::Out pipe_stream;
+
+#ifdef HAVE_STREAM_IMBUE
+	      pipe_stream.imbue(std::locale::classic());
+#endif // HAVE_STREAM_IMBUE
+
 	      pipe_stream.attach(write_fd);
 	      pipe_stream << existing_lock_pid << std::endl;
 	      pipe_stream.close();
@@ -1036,6 +1075,11 @@ bool is_program_running(void) {
   if (count < 2 && new_lock_fd >= 0)  {// we have successfully created a lock file - now write our pid to it
 
     Attachable::Out new_lock;
+
+#ifdef HAVE_STREAM_IMBUE
+    new_lock.imbue(std::locale::classic());
+#endif // HAVE_STREAM_IMBUE
+
     new_lock.attach(new_lock_fd);
     new_lock << std::setfill('0') << std::setw(10) << getpid() << std::endl;
     new_lock.close();
@@ -1089,11 +1133,18 @@ void atexit_cleanup(void) {
   // clean up the receive_directory if necessary
   clean_up_receive_dir();
 
-  // call up a shell to dispense with any stray viewing files
-  // (these may exist if the program is exited when a fax is still being viewed)
-  // whether this executes or not is pretty optional so whilst I would normally
-  // avoid calling system(), it is acceptable in this limited context (IMO)!
-  std::system("rm -f /tmp/efax-gtk-view.*");
+  // call up a shell to dispense with any stray viewing or printing files
+  // (these may exist if the program crashes, or is exited when a fax is still
+  // being viewed) whether this executes or not is pretty optional so whilst
+  // I would normally avoid calling system(), it is acceptable in this limited
+  // context (IMO)
+  std::string command_line("rm -f ");
+  command_line += prog_config.working_dir;
+  command_line += "/efax-gtk-view-* ";
+  command_line += prog_config.working_dir;
+  command_line += "/efax-gtk-print-*";
+
+  std::system(command_line.c_str());
 }
 
 void clean_up_receive_dir(void) {
