@@ -6,6 +6,14 @@ COPYING distributed with the source files.
 
 */
 
+// something in gtkprintjob.h clashes with the C++ headers with
+// libstdc++-5 and it needs to be included here to obtain a
+// clean compile
+#include <gtk/gtkversion.h>
+#if GTK_CHECK_VERSION(2,10,0)
+#include <gtk/gtkprintjob.h>
+#endif
+
 #include <unistd.h>
 #include <signal.h>
 #include <sys/stat.h>
@@ -19,13 +27,19 @@ COPYING distributed with the source files.
 #include <cstring>
 
 #ifdef HAVE_OSTREAM
+#include <ios>
 #include <ostream>
 #endif
 
 #ifdef HAVE_ISTREAM
+#include <ios>
 #include <istream>
 #endif
 
+#ifdef HAVE_STREAM_IMBUE
+#include <locale>
+#endif
+
 #include <gtk/gtkcontainer.h>
 #include <gtk/gtkbox.h>
 #include <gtk/gtkhbox.h>
@@ -54,8 +68,8 @@ COPYING distributed with the source files.
 #include <gtk/gtkimage.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkiconfactory.h>
+#include <gtk/gtkenums.h>
 #include <gtk/gtkmain.h>
-#include <gtk/gtkversion.h>
 
 #include <pango/pango-layout.h>
 
@@ -68,14 +82,10 @@ COPYING distributed with the source files.
 #include <glib/glist.h>
 #include <glib/gmain.h>
 
-// a guess for the future!
-#if GTK_MAJOR_VERSION > 2
-#define EFAX_GTK_USE_ICON_THEME 1
-// a pick for the present
-#elif GTK_MINOR_VERSION >= 4
-#define EFAX_GTK_USE_ICON_THEME 1
+#if GTK_CHECK_VERSION(2,4,0)
+#  define EFAX_GTK_USE_ICON_THEME 1
 #else
-#undef EFAX_GTK_USE_ICON_THEME
+#  undef EFAX_GTK_USE_ICON_THEME
 #endif
 
 #ifdef EFAX_GTK_USE_ICON_THEME
@@ -101,6 +111,7 @@ COPYING distributed with the source files.
 
 #define TIMER_INTERVAL 200              // number of milliseconds between timer events for the main timer
 #define LOGFILE_TIMER_INTERVAL 60000    // number of milliseconds between flushing of logfile
+#define LOGFILE_MARK_COUNT 15*60000/LOGFILE_TIMER_INTERVAL // to mark the logfile at 15 minute intervals
 
 extern "C" void close_signalhandler(int);
 static volatile sig_atomic_t close_flag = false;
@@ -797,17 +808,32 @@ void MainWindow::get_window_settings(void) {
   int width = 0;
   int height = 0;
   int socket_val = 0;
+  int received_list_sort_type = -1;
+  int sent_list_sort_type = -1;
 
   std::string file_name(prog_config.working_dir + "/" MAINWIN_SAVE_FILE);
   std::ifstream file(file_name.c_str(), std::ios::in);
 
   if (!file) write_error("Can't get mainwindow settings from file\n");
   else {
-    file >> width >> height >> socket_val;
+
+#ifdef HAVE_STREAM_IMBUE
+    file.imbue(std::locale::classic());
+#endif // HAVE_STREAM_IMBUE
+
+    file >> width >> height >> socket_val
+	 >> received_list_sort_type >> sent_list_sort_type;
 
     if (width > 0 && height > 0) {
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(socket_button_p), socket_val);
       gtk_window_set_default_size(get_win(), width, height);
+      if ((received_list_sort_type == GTK_SORT_ASCENDING
+	   || received_list_sort_type == GTK_SORT_DESCENDING)
+	  && (sent_list_sort_type == GTK_SORT_ASCENDING
+	      || sent_list_sort_type == GTK_SORT_DESCENDING)) {
+	FaxListManager::set_received_list_sort_type(GtkSortType(received_list_sort_type));
+	FaxListManager::set_sent_list_sort_type(GtkSortType(sent_list_sort_type));
+      }
     }
   }
 }
@@ -820,13 +846,22 @@ void MainWindow::save_window_settings(void) {
   if (!file) write_error("Can't save mainwindow settings to file\n");
   else {
 
-    int width;
-    int height;
+#ifdef HAVE_STREAM_IMBUE
+    file.imbue(std::locale::classic());
+#endif // HAVE_STREAM_IMBUE
+
+    int width = 0;
+    int height = 0;
     gtk_window_get_size(get_win(), &width, &height);
 
     int socket_val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(socket_button_p));
     
-    file << width << ' ' << height << ' ' << socket_val << std::endl;
+    file << width << ' '
+	 << height << ' '
+	 << socket_val << ' '
+	 << (int)FaxListManager::get_received_list_sort_type() << ' '
+	 << (int)FaxListManager::get_sent_list_sort_type() << ' '
+	 << std::endl;
   }
 }
 
@@ -1405,6 +1440,7 @@ void MainWindow::translations_impl(void) {
   message += gettext("German - Steffen Wagner\n");
   message += gettext("Swedish - Daniel Nylander\n");
   message += gettext("Catalan - Jordi Sayol Salomo\n");
+  message += gettext("Traditional Chinese - Wei-Lun Chao\n");
 
   new InfoDialog(message.c_str(), gettext("efax-gtk: Translations"),
 		 GTK_MESSAGE_INFO, get_win());
@@ -1431,11 +1467,8 @@ void MainWindow::get_longest_button_text(void) {
   int height;
   int max_width;
 
-  // create a button to work on
+  // create a button to work on - the GobjHandle object will claim ownership of the button
   GobjHandle<GtkWidget> button_h(gtk_button_new());
-  // simulate its placement in a container
-  g_object_ref(G_OBJECT(button_h.get()));
-  gtk_object_sink(GTK_OBJECT(button_h.get()));
 
   for (temp_iter = text_vec.begin(), max_width_iter = text_vec.begin(),
 	 width = 0, height = 0, max_width = 0;
@@ -1456,7 +1489,7 @@ void MainWindow::get_longest_button_text(void) {
 void MainWindow::tray_icon_left_clicked_slot(void) {
 
   GtkWidget* mainwin_p = GTK_WIDGET(get_win());
-  // test Gtk::Window::is_visible() as well as the 'obscured' variable - if we call
+  // test GTK_WIDGET_VISIBLE() as well as the 'obscured' variable - if we call
   // hide() on the window it does not count as a visibility notify event!
   if (GTK_WIDGET_VISIBLE(mainwin_p)
       && !obscured
@@ -1502,11 +1535,8 @@ void MainWindow::style_set_impl(void) {
   // the main task is to set the buttons in MainWindow
   int width = 0;
   int height = 0;
-  // create a button to work on
+  // create a button to work on - the GobjHandle object will claim ownership of the button
   GobjHandle<GtkWidget> button_h(gtk_button_new());
-  // simulate its placement in a container
-  g_object_ref(G_OBJECT(button_h.get()));
-  gtk_object_sink(GTK_OBJECT(button_h.get()));
 
   GtkStyle* style_p = gtk_widget_get_style(GTK_WIDGET(get_win()));
   gtk_widget_set_style(button_h, style_p);
@@ -1649,8 +1679,18 @@ void MainWindow::strip(std::string& text) {
 
 void MainWindow::start_pipe_thread(void) {
 
-  // needed for pipe_thread()
+  // needed for pipe_thread() - we need to get the program PID here, as
+  // under the linuxthreads implementation of pthreads, getpid() will
+  // yield a different value for different threads in the same process
+  // and we need to use the PID yielded by the main (GUI) thread
   prog_pid = getpid();
+  // read ProgConfig::working_dir in this thread - if we read it in
+  // MainWindow::pipe_thread() we would have to protect accesses with
+  // the ProgConfig::mutex_p global mutex since not all implementations
+  // of std::string provide concurrent read access, and it is best to
+  // avoid use of the global mutex where that can easily be done
+  prog_fifo_name = prog_config.working_dir;
+  prog_fifo_name += PIPE_NAME;
 
   // block off the signals for which we have set handlers so that the pipe
   // thread does not receive the signals, otherwise we will have memory synchronisation
@@ -1676,24 +1716,26 @@ void MainWindow::start_pipe_thread(void) {
 
 void MainWindow::pipe_thread(void) {
 
-  std::string fifo_name(prog_config.working_dir);
-  fifo_name += PIPE_NAME;
-
-  if(access(fifo_name.c_str(), F_OK) == -1) {
+  if(access(prog_fifo_name.c_str(), F_OK) == -1) {
     // fifo doesn't exit - create it
-    if (mkfifo(fifo_name.c_str(), 0666) != 0) {
+    if (mkfifo(prog_fifo_name.c_str(), 0666) != 0) {
       write_error("Error creating fifo in MainWindow::pipe_thread()\n");
       return;
     }
   }
 
   for (;;) {
-    int read_fd = open(fifo_name.c_str(), O_RDONLY);
+    int read_fd = open(prog_fifo_name.c_str(), O_RDONLY);
     if(read_fd == -1) {
       write_error("Error opening fifo in MainWindow::pipe_thread()\n");
       return;
     }
     Attachable::In pipe_stream;
+
+#ifdef HAVE_STREAM_IMBUE
+    pipe_stream.imbue(std::locale::classic());
+#endif // HAVE_STREAM_IMBUE
+
     pipe_stream.attach(read_fd);
     
     pid_t pid = 0;
@@ -1731,7 +1773,7 @@ namespace { // callback for internal use only
 
 gboolean MessageTextCB::message_text_logfile_timer(void* data) {
 
-  static_cast<MessageText*>(data)->logfile.flush();
+  static_cast<MessageText*>(data)->logfile_timer_impl();
   return true; // we want a multi-shot timer
 }
 
@@ -1739,7 +1781,8 @@ gboolean MessageTextCB::message_text_logfile_timer(void* data) {
 
 
 MessageText::MessageText(void):
-             MainWidgetBase(gtk_scrolled_window_new(0, 0)) {
+                       MainWidgetBase(gtk_scrolled_window_new(0, 0)),
+                       logfile_count(0) {
 
   text_view_p = gtk_text_view_new();
   gtk_container_add(GTK_CONTAINER(get_main_widget()), text_view_p);
@@ -1757,6 +1800,7 @@ MessageText::MessageText(void):
   red.red = static_cast<guint16>(0.8 * 65535.0);
   red.green = 0;
   red.blue = 0;
+  // GtkTextTags are not owned by a GTK+ container when passed to it so use a GobjHandle
   red_tag_h = GobjHandle<GtkTextTag>(gtk_text_tag_new(0));
 
   GdkColormap* colormap_p = gtk_widget_get_default_colormap();
@@ -1903,6 +1947,7 @@ void MessageText::reset_logfile(void) {
     logfile.clear();
 
     logfile_name = "";
+    logfile_count = 0;
     // and now disconnect the old timer connection
     g_source_remove(timer_tag);
   }
@@ -1940,6 +1985,7 @@ void MessageText::reset_logfile(void) {
 	timer_tag = g_timeout_add(LOGFILE_TIMER_INTERVAL,
 				  MessageTextCB::message_text_logfile_timer, this);
       }
+      logfile_count = 0;
     }
     catch (Utf8::ConversionError&) {
       write_error("UTF-8 conversion error in MessageText::reset_logfile()\n");
@@ -1957,6 +2003,27 @@ void MessageText::reset_logfile(void) {
   logfile_name = Utf8::filename_from_utf8(prog_config.logfile_name);
 }
 
+void MessageText::logfile_timer_impl(void) {
+  
+  logfile_count++;
+  if (logfile_count >= LOGFILE_MARK_COUNT) {
+    logfile_count = 0;
+
+    struct std::tm* time_p;
+    std::time_t time_count;
+    std::time(&time_count);
+    time_p = std::localtime(&time_count);
+      
+    const char date_description_format[] = "%H%M %Z %d %b %Y";
+    const int max_description_datesize = 126;
+    char date_description[max_description_datesize];
+    std::strftime(date_description, max_description_datesize, date_description_format, time_p);
+
+    logfile << "\n**** " << date_description << " ****\n" << std::endl;
+  }
+  logfile.flush();
+}
+
 void MessageText::cleanify(std::string& message) {
 
   std::string::size_type chunk_start = 0;
