@@ -122,9 +122,15 @@ ParmsMessages::ParmsMessages(void): SettingsMessagesBase(extra_parms + 1) {
 
 PrintMessages::PrintMessages(void): SettingsMessagesBase(popup + 1) {
 
+  messages[gtkprint] = gettext("If this is checked, the program will use the GTK+ print system "
+			       "rather than the command line argument specified below to print "
+			       "faxes");
+  captions[gtkprint] = gettext("efax-gtk help: Use GTK+ print system");
+
   messages[command] = gettext("This is the command which prints and takes postscript input "
 			      "on standard input.  If none is specified, the program will default "
-			      "to `lpr'");
+			      "to `lpr'.  This has no effect if the \"Use GTK+ Print System\" button "
+			      "is checked");
   captions[command] = gettext("efax-gtk help: Print Program");
 
   messages[shrink] = gettext("This will determine the extent to which a printed page will "
@@ -138,7 +144,8 @@ PrintMessages::PrintMessages(void): SettingsMessagesBase(popup + 1) {
 			    "printing whenever the \"Print selected fax\" button is pressed in the "
 			    "Received Faxes list or Sent Faxes list.  Uncheck the box if you don't "
 			    "want a dialog (if you have specified an external print manager such as "
-			    "'kprinter' in the Print Program box, then you will not want this checked).");
+			    "'kprinter' in the Print Program box, then you will not want this checked).  "
+			    "This has no effect if the \"Use GTK+ Print System\" button is checked");
   captions[popup] = gettext("efax-gtk help: Popup");
 }
 
@@ -165,9 +172,9 @@ SockMessages::SockMessages(void): SettingsMessagesBase(client_address + 1) {
   messages[popup] = gettext("This specifies whether a pop-up dialog is to appear whenever a fax "
 			    "is received by the socket server from the print system.  Check the box "
 			    "if you want a dialog (this has no effect unless you have also checked "
-			    "the box for the socket server to run).  If the program is inactive (for "
-			    "example, it is not standing by to receive faxes) the fax can be sent "
-			    "directly from this dialog.");
+			    "the box for the socket server to run).  If the program is inactive or "
+			    "is standing by to receive faxes, the fax can be sent directly from "
+			    "this dialog");
   captions[popup] = gettext("efax-gtk help: Popup");
 
   messages[port] = gettext("This specifies the port number on which the socket server is to "
@@ -178,11 +185,13 @@ SockMessages::SockMessages(void): SettingsMessagesBase(client_address + 1) {
 				     "host name will be allowed to make a connection to the socket. Unless you have "
 				     "unusual requirements (that is, unless the CUPS or lpd daemons are running on "
 				     "another machine), you should choose localhost.  If you want other machines to "
-				     "be able to connect, pick \"other\" and specify the fully qualified hostnames of "
-				     "the computers which may connect (separating different hostnames by spaces, but "
-				     "you do not need to specify localhost and your machine's hostname, as those are "
-				     "always permitted).  You should run efax-gtk behind a firewall if you choose "
-				     "\"other\".");
+				     "be able to connect, pick \"other\" and specify the hostnames of the computers "
+				     "which may connect (separating different hostnames by spaces, but you do not "
+				     "need to specify localhost and your machine's hostname, as those are always "
+				     "permitted).  Such hostnames may be specified as an ordinary host name or in "
+				     "numeric dot notation.  Numeric dot notation may have * as a trailing wildcard, "
+				     "so for example 10.1.* will permit access for addresses 10.1.0.0 to 10.1.255.255.  "
+				     "You should run efax-gtk behind a firewall if you choose \"other\".");
   captions[client_address] = gettext("efax-gtk help: Socket connections allowed");
 }
 
@@ -194,7 +203,10 @@ ReceiveMessages::ReceiveMessages(void): SettingsMessagesBase(exec + 1) {
 
   messages[exec] = gettext("This specifies whether a program or script is to be executed when a "
 			   "fax is received from a modem.  Check the box if you want to execute a "
-			   "program, and enter the program name.");
+			   "program, and enter the program name.  efax-gtk comes with two scripts, "
+                           "print_fax and mail_fax, which can be used automatically to print a "
+                           "received fax or to mail it to a user (so to mail a fax you can enter "
+                           "`mail_fax' here).  See the README file for further details.");
   captions[exec] = gettext("efax-gtk help: Execute program when fax received");
 }
 
