@@ -60,7 +60,7 @@ public:
 
 class PrintMessages: private SettingsMessagesBase {
 public:
-  enum {command = 0, shrink, popup};
+  enum {gtkprint = 0, command, shrink, popup};
   PrintMessages(void);
 
   using SettingsMessagesBase::get_caption;
