@@ -40,6 +40,7 @@ COPYING distributed with the source files.
 #include "utils/utf8_utils.h"
 
 #ifdef HAVE_OSTREAM
+#include <ios>
 #include <ostream>
 #endif
 
@@ -139,7 +140,7 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
 				 GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
 
   // create the tree model and put it in a GobjHandle to handle its
-  // lifetime, because it is derived from GObject but not GtkObject
+  // lifetime, because it is not owned by a GTK+ container
   list_store_h =
     GobjHandle<GtkTreeModel>(GTK_TREE_MODEL(gtk_list_store_new(ModelColumns::cols_num,
 							       G_TYPE_STRING, G_TYPE_STRING)));
@@ -176,6 +177,7 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
 
   GtkWidget* image_p;
   { // scope block for the GobjHandle
+    // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
     GobjHandle<GdkPixbuf> pixbuf_h(gdk_pixbuf_new_from_xpm_data(view_xpm));
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(view_button_p), image_p);
@@ -185,6 +187,7 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
   gtk_widget_set_sensitive(view_button_p, false);
 
   { // scope block for the GobjHandle
+    // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
     GobjHandle<GdkPixbuf> pixbuf_h(gdk_pixbuf_new_from_xpm_data(remove_xpm));
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(remove_button_p), image_p);
@@ -195,6 +198,7 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
 
   GtkWidget* send_hbox_p = gtk_hbox_new(false, 0);
   { // scope block for the GobjHandle
+    // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
     GobjHandle<GdkPixbuf> pixbuf_h(gdk_pixbuf_new_from_xpm_data(send_fax_xpm));
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     GtkWidget* label_p = gtk_label_new(gettext("Enter selected fax to send"));
