@@ -19,7 +19,7 @@ COPYING distributed with the source files.
 
 #include <sigc++/sigc++.h>
 
-#include "window.h"
+#include "utils/window.h"
 
 
 namespace { // we put the function in anonymous namespace in socket_notify.cpp
