@@ -19,6 +19,7 @@ COPYING distributed with the source files.
 #include <arpa/inet.h>
 #include <unistd.h>
 #include <netdb.h>
+#include <netinet/in.h>
 #include <errno.h>
 #include <pthread.h>
 
@@ -26,9 +27,14 @@ COPYING distributed with the source files.
 #include "utils/shared_handle.h"
 
 #ifdef HAVE_OSTREAM
+#include <ios>
 #include <ostream>
 #endif
 
+#ifdef HAVE_STREAM_IMBUE
+#include <locale>
+#endif
+
 #ifdef ENABLE_NLS
 #include <libintl.h>
 #endif
@@ -120,8 +126,8 @@ void SocketServer::socket_thread(void) {
     serve_address.sin_family = AF_INET;
     if (other_sock_client_address) serve_address.sin_addr.s_addr = htonl(INADDR_ANY);
     else {
-      long localhost_address = inet_addr("127.0.0.1");
-      if (localhost_address > -1) serve_address.sin_addr.s_addr = localhost_address;
+      in_addr_t localhost_address = inet_addr("127.0.0.1");
+      if (localhost_address != (in_addr_t)(-1)) serve_address.sin_addr.s_addr = localhost_address;
       else {
 	write_error("Cannot create network address for localhost\n");
 	socket_error_notify();
@@ -157,6 +163,7 @@ bool SocketServer::accept_on_client(void) {
   bool return_val = false;
 
   sockaddr_in connect_address;
+  std::memset(&connect_address, 0, sizeof(connect_address));
   socklen_t client_len = sizeof(connect_address);
 
   // the calls to pthread_testcancel() are a workaround for Linuxthreads -
@@ -184,7 +191,7 @@ bool SocketServer::accept_on_client(void) {
     std::memcpy(tempfile_h.get(), filename.c_str(), size); // this will include the terminating '\0' in the copy
     int file_fd = mkstemp(tempfile_h.get());
 
-    if (file_fd < 0) {
+    if (file_fd == -1) {
       write_error("Can't open temporary file for socket server\n"
 		  "This fax will not be processed.  Is the disk full?\n");
       return_val = false;
@@ -426,6 +433,11 @@ void SocketServer::save_queued_faxes(void) {
     write_error(message.c_str());
   }
   else {
+
+#ifdef HAVE_STREAM_IMBUE
+    fileout.imbue(std::locale::classic());
+#endif // HAVE_STREAM_IMBUE
+
     FilenamesList::const_iterator iter;
     for (iter = filenames_s->begin(); iter != filenames_s->end(); ++iter) {
       fileout << iter->first << '?' << iter->second << '\n';
@@ -468,63 +480,95 @@ bool SocketServer::is_valid_peer(const sockaddr_in& address) {
 
   bool return_val = false;
   hostent* hostinfo_p;
+  std::vector<std::string> peer_names;
 
-  // gethostbyaddr is not guaranteed by IEEE Std 1003.1 to be thread safe -
+  // gethostbyaddr() is not guaranteed by IEEE Std 1003.1 to be thread safe -
   // that doesn't matter as this program only uses it in this thread
-  if ((hostinfo_p = gethostbyaddr((const char*)&address.sin_addr.s_addr,
-				  sizeof(address.sin_addr.s_addr), AF_INET)) == 0) {
-    write_error("Cannot get hostname of peer\n");
+  if ((hostinfo_p = gethostbyaddr((const char*)&address.sin_addr,
+				  sizeof(address.sin_addr), AF_INET)) == 0) {
+    // inet_ntoa() is not guaranteed by IEEE Std 1003.1 to be thread safe -
+    // that doesn't matter as this program only uses it in this thread
+    const char* name_p = inet_ntoa(address.sin_addr);
+    if (name_p) peer_names.push_back(name_p);
+    else write_error("Cannot get hostname or numeric address of peer\n");
   }
 
   else {
-    // we need to store the value of hostinfo_p->h_name,
-    // as gethostbyaddr() is called again below
-    std::string peer_hostname(hostinfo_p->h_name);
-    std::vector<std::string> valid_names;
-    valid_names.push_back("localhost");          // localhost is always valid
-    // include the fully qualified localhost address
+    // store the values in hostinfo_p,
+    peer_names.push_back(hostinfo_p->h_name);
+    // load in any aliases
+    char** addrs_pp;
+    for (addrs_pp = hostinfo_p->h_aliases; *addrs_pp; addrs_pp++) {
+      peer_names.push_back(*addrs_pp);
+    }
+    // now load in the address(es) of the peer in numeric dot notation
+    // inet_ntoa() is not guaranteed by IEEE Std 1003.1 to be thread safe -
+    // that doesn't matter as this program only uses it in this thread
+    for (addrs_pp = hostinfo_p->h_addr_list; *addrs_pp; addrs_pp++) {
+      const char* name_p = inet_ntoa(*((in_addr*)*addrs_pp));
+      if (name_p) peer_names.push_back(name_p);
+    }
+  }
+
+  std::vector<std::string> valid_names;
+  valid_names.push_back("localhost");           // localhost is always valid
+  valid_names.push_back("127.0.0.1");           // ditto
+
+  char my_hostname_p[256];
+  if (gethostname(my_hostname_p, 256) < 0) {
+    write_error("Cannot get our hostname\n");
+  }
+  else {
+    my_hostname_p[255] = 0; // make sure it is null terminated
+    valid_names.push_back(my_hostname_p);       // our own hostname is also always valid
+    // gethostbyname is not guaranteed by IEEE Std 1003.1 to be thread safe -
+    // that doesn't matter as this program only uses it in this thread
     hostent* my_info_p;
-    long localhost_address = inet_addr("127.0.0.1");
-    if (localhost_address == -1) {
-	write_error("Cannot create network address for localhost\n");
-    }
-    else if ((my_info_p = gethostbyaddr((const char*)&localhost_address,
-					sizeof(address.sin_addr.s_addr), AF_INET)) == 0) {
-      write_error("Cannot get the fully qualified localhost name\n");
+    if ((my_info_p = gethostbyname(my_hostname_p)) == 0) {
+      write_error("Cannot get our fully qualified hostname\n");
     }
     else {
-      valid_names.push_back(my_info_p->h_name);  // and our fq localhost name is also valid
+      valid_names.push_back(my_info_p->h_name); // and our fq hostname is also valid
     }
+  }
 
-    char my_hostname_p[256];
-    if (gethostname(my_hostname_p, 256) < 0) {
-      write_error("Cannot get our hostname\n");
-    }
-    else {
-      my_hostname_p[255] = 0; // make sure it is null terminated
-      valid_names.push_back(my_hostname_p);       // our own hostname is also always valid
-      // gethostbyname is not guaranteed by IEEE Std 1003.1 to be thread safe -
-      // that doesn't matter as this program only uses it in this thread
-      if ((my_info_p = gethostbyname(my_hostname_p)) == 0) {
-	write_error("Cannot get our fully qualified hostname\n");
-      }
-      else {
-	valid_names.push_back(my_info_p->h_name); // and our fq hostname is also valid
-      }
-    }
+  // and now add the permitted clients from the settings dialog
 
-    // and now add the permitted clients from the settings dialog
-
-    // lock the Prog_config object to stop it being modified or accessed in the initial
-    // (GUI) thread while we are accessing it here
-    Thread::Mutex::Lock lock(*prog_config.mutex_p);
-    std::copy(prog_config.permitted_clients_list.begin(),
-	      prog_config.permitted_clients_list.end(),
-	      std::back_inserter(valid_names));
-    std::vector<std::string>::const_iterator iter;
-    for (iter = valid_names.begin(); !return_val && iter != valid_names.end(); ++iter) {
-      if (*iter == peer_hostname) return_val = true;
+  // lock the Prog_config object to stop it being modified or accessed in the initial
+  // (GUI) thread while we are accessing it here
+  Thread::Mutex::Lock lock(*prog_config.mutex_p);
+  std::copy(prog_config.permitted_clients_list.begin(),
+	    prog_config.permitted_clients_list.end(),
+	    std::back_inserter(valid_names));
+  std::vector<std::string>::const_iterator peer_names_iter;
+  std::vector<std::string>::const_iterator valid_names_iter;
+  for (peer_names_iter = peer_names.begin();
+       !return_val && peer_names_iter != peer_names.end();
+       ++peer_names_iter) {
+    for (valid_names_iter = valid_names.begin();
+	 !return_val && valid_names_iter != valid_names.end();
+	 ++valid_names_iter) {
+      return_val = addr_compare(*valid_names_iter, *peer_names_iter);
     }
   }
   return return_val;
 }
+
+bool SocketServer::addr_compare(const std::string& wild_str, const std::string& in_str) {
+
+  // check pre-conditions
+  if (wild_str.empty()) return false;
+
+  if (wild_str[wild_str.size() - 1] == '*'
+      && wild_str.find_first_not_of("0123456789.*") == std::string::npos) {
+
+    // take trailing '*' characters out of the comparison
+    std::string::size_type length = wild_str.size();
+    do {
+      length--;
+    } while (length > 0 && wild_str[length - 1] == '*');
+
+    return wild_str.substr(0, length) == in_str.substr(0, length);
+  }
+  return wild_str == in_str;
+}
