@@ -23,6 +23,7 @@ COPYING distributed with the source files.
 #include "utils/gobj_handle.h"
 
 #ifdef HAVE_OSTREAM
+#include <ios>
 #include <ostream>
 #endif
 
@@ -198,6 +199,7 @@ TrayItem::TrayItem(GtkWindow* window_p): prog_window_p(window_p),
   // now create and register the tray icon container
   event_box_p = gtk_event_box_new();
   { // scope block for the GobjHandle
+    // GdkPixbufs are not owned by a GTK+ container when passed to it so use a GobjHandle
     GobjHandle<GdkPixbuf> pixbuf_h(gdk_pixbuf_scale_simple(prog_config.window_icon_h,
 							   22, 22, GDK_INTERP_HYPER));
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
