@@ -22,14 +22,20 @@ License as follows:
 
 */
 
-#include <gtk/gtkobject.h>
 #include <glib-object.h>
+#include <gtk/gtkobject.h>
+#include <gtk/gtkversion.h>
 
 #include "widget.h"
 
 MainWidgetBase::MainWidgetBase(GtkWidget* widget_p): g_widget_p(widget_p) {
+
+#if GTK_CHECK_VERSION(2,9,0)
+  g_object_ref_sink(G_OBJECT(g_widget_p));
+#else
   g_object_ref(G_OBJECT(g_widget_p));
   gtk_object_sink(GTK_OBJECT(g_widget_p));
+#endif
 }
 
 MainWidgetBase::~MainWidgetBase(void) {
